from elasticsearch import Elasticsearch
from nltk.corpus import stopwords
from nltk import word_tokenize
import gensim

def results(es, es_index, query_str):
    """Return results for query_str in es_index"""
    # TODO
    return es.search(index=es_index, size=100, body={"query": {
        "query_string" : {
            "default_field" : "text",
            "query" : query_str# "\"" + query_str + "\""
        }
    }})


def hit_title(hit):
    """Return title of hit"""
    # TODO
    return hit["_source"]["title"]


def hit_text(hit):
    """Return text of hit"""
    # TODO
    return hit["_source"]["text"]


def context(needle, haystack, window=50, highlight_leftright=("","")):
    start = haystack.lower().find(needle.lower())
    if -1 == start:
        return haystack
    end = start + len(needle)
    left = haystack[start-window:start]
    middle = highlight_leftright[0] + haystack[start: end] + highlight_leftright[1]
    right = haystack[end:end+window]
    return left + middle + right


def main():
    es = Elasticsearch()
    question_file = "/home/ben/data/levy_zero_shot/relation_splits/question_templates.txt"
    out_file = "/home/ben/data/levy_zero_shot/relation_splits/question_template_sentences"
    stopWords = set(stopwords.words('english'))
    stopWords.add("xxx")

    print("loading word vectors...")
    word_vectors_file="/home/ben/data/glove/glove.6B.300d.txt"
    m=gensim.models.KeyedVectors.load_word2vec_format(word_vectors_file)
    print("... done!")


    with open(out_file, "w") as f_out:
        with open(question_file) as f:
            for line in f:
                line = line.strip()
                words = word_tokenize(line)
                words = [w.lower() for w in words if w.isalpha() and w.lower() in m.vocab]
                words = [w for w in words if w not in stopWords]
                if not words:
                    continue
                print("===")
                print(line)
                query_str = " ".join(words + [w[0] for w in m.wv.most_similar(words,None,5) if w[0].isalpha()])
                print(query_str)
                res = results(es, 'wiki-sentences', query_str)
                print("Got %d Hits:" % res['hits']['total'])
                for hit in res['hits']['hits']:
                    title = hit_title(hit)
                    text = hit_text(hit).strip()
                    if "\n" in text or len(word_tokenize(text)) < 5 or len(word_tokenize(text)) > 60:
                        continue
                    print("Article: " + title)
                    print(text)
                    print()
                    f_out.write(line + "\t" + text + "\n")



#    while True:
#        query_str = input("Query: ").strip()
#        if not query_str:
#            break
#        res = results(es, 'wiki-sentences', query_str)
#        print("Got %d Hits:" % res['hits']['total'])
#        for hit in res['hits']['hits']:
#            title = hit_title(hit)
#            text = hit_text(hit)
#            print("Article: " + title)
#            print("..." + context(needle=query_str, haystack=text, highlight_leftright = ("[","]")).replace("\n", " ") + "...")
#            print()

# lib/elasticsearch-5.2.1/bin/elasticsearch
if __name__ == "__main__":
    main()
