import argparse

def matches(answer1, answer2, strict=True):
    if strict:
        return answer1 == answer2
    else:
        if answer1 == answer2:
            return True
        elif answer1=="" or answer2=="":
            return False
        elif answer1.startswith(answer2) or answer1.endswith(answer2) or \
            answer2.startswith(answer1) or answer2.endswith(answer1):
            return True
        else:
            return False

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--key',type=str)
    parser.add_argument('--prediction',type=str)
    args = parser.parse_args()

    key_file = open(args.key)
    pred_file = open(args.prediction)

    key_answer_tuples = set()
    response_answer_tuples = set()

    instance_relevant_retrieved = 0
    instance_relevant = 0
    instance_retrieved = 0
    instance_correct = 0
    instance_all = 0

    for key_line in key_file:
        parts = key_line.strip().split("\t")
        query = parts[1]
        relation = parts[2]
        key_answer = parts[3].replace("</s>", "").strip()
        pred_answer = pred_file.readline().replace("</s>", "").strip()
        instance_all += 1

        if len(pred_answer) > 0:
            instance_retrieved += 1
            response_answer_tuples.add((query, relation, pred_answer))

        if len(key_answer) > 0:
            instance_relevant += 1
            key_answer_tuples.add((query, relation, key_answer))

        if matches(key_answer, pred_answer, strict=True):
            instance_correct += 1
            if len(pred_answer) > 0:
                instance_relevant_retrieved += 1

    instance_acc = instance_correct / instance_all
    instance_prec = instance_relevant_retrieved / instance_retrieved
    instance_rec = instance_relevant_retrieved / instance_relevant
    instance_f1 = 2 * instance_prec * instance_rec / (instance_prec + instance_rec)
    print("Instance-level evaluation:")
    print("Precision: %.4f" % instance_prec)
    print("Recall: %.4f" % instance_rec)
    print("F1: %.4f" % instance_f1)
    print("Accuracy: %.4f" % instance_acc)

    relevant_retrieved = set.intersection(response_answer_tuples, key_answer_tuples)
    tuple_prec = len(relevant_retrieved) / len(response_answer_tuples)
    tuple_rec = len(relevant_retrieved) / len(key_answer_tuples)
    tuple_f1 = 2 * tuple_prec * tuple_rec / (tuple_prec + tuple_rec)
    print("===\nTuple-level evaluation:")
    print("Precision: %.4f" % tuple_prec)
    print("Recall: %.4f" % tuple_rec)
    print("F1: %.4f" % tuple_f1)
