import numpy

SEED = 0
# fix random seed for reproducibility
numpy.random.seed(SEED)

from refactored import models
from refactored.models import PointerModel

import argparse

class SlotFillerModel(PointerModel):
    def encode_sequence(self):
        return models.encode_sequence_with_cnn(self.max_length, self.word_to_index, self.vocab_size, self.rellist)

def arg_parser():
    parser = argparse.ArgumentParser(description='Train and evaluate a slot filler model.')
    parser.add_argument('--train', type=str, help='Path to training data', default=None)
    parser.add_argument('--dev', type=str, help='Path to development data', default=None)
    parser.add_argument('--test', type=str, help='Path to test data', default=None)
    parser.add_argument('--prediction', type=str, help='Prediction for test data', default=None)
    parser.add_argument('--model', type=str, help='If train and dev give: Save model here. Otherwise, load from here.', default="")
    parser.add_argument('--vocab', type=str, help='If train and dev give: Save vocab here. Otherwise, load from here.', default="")
    parser.add_argument('--rels', type=str, help='relations to consider', default="/mounts/Users/cisintern/beroth/data/tac/rellist2013")
    args = parser.parse_args()
    return args

if __name__ == "__main__":
    args = arg_parser()
    print("Parameter summary:")
    for key in vars(args):
        print(key, ":", vars(args)[key])
        if vars(args)[key] == "":
            print("Argument", key, "must not be empty")
            raise Exception()

    m = SlotFillerModel(trainpath=args.train, devpath=args.dev, testpath=args.test, vocabpath=args.vocab, relfile=args.rels)

    config = {"dummy_arg":True}
    m.model_entity_recognition(config)
    if args.train and args.dev:
        m.train_entity_recognition(args.model)
    else:
        m.model.load_weights(args.model)
    if args.test and args.prediction:
        m.predict_dataset_fast("test", args.prediction)