from keras import backend as K
from keras import activations
from keras.engine.topology import Layer
import numpy as np

class SelfAttention(Layer):

    def __init__(self, output_dim, **kwargs):
        self.output_dim = output_dim
        super(SelfAttention, self).__init__(**kwargs)

    def build(self, input_shape):
        ## Create a trainable weight variables for this layer.
        # query
        self.w_q = self.add_weight(name='w_q',
                                   shape=(input_shape[-1], self.output_dim),
                                   initializer='glorot_uniform',
                                   trainable=True)
        # key
        self.w_k = self.add_weight(name='w_k',
                                   shape=(input_shape[-1], self.output_dim),
                                   initializer='glorot_uniform',
                                   trainable=True)
        # value
        self.w_v = self.add_weight(name='w_v',
                                   shape=(input_shape[-1], self.output_dim),
                                   initializer='glorot_uniform',
                                   trainable=True)

        self.scaling_factor = K.sqrt(1.0/self.output_dim)
        super(SelfAttention, self).build(input_shape)  # Be sure to call this somewhere!

    def call(self, input_seq):
        query_seq = K.dot(input_seq, self.w_q)
        key_seq = K.dot(input_seq, self.w_k)
        value_seq = K.dot(input_seq, self.w_v)

        weights = K.batch_dot(query_seq, K.permute_dimensions(key_seq, (0,2,1))) # TODO: batch_dot? reshape?
        weights = activations.softmax(weights * self.scaling_factor) # TODO: Rescale weights or rely on w_q and w_k?
        reweighted_values = K.batch_dot(weights, value_seq)
        return reweighted_values

    def compute_output_shape(self, input_shape):
        return (input_shape[0], input_shape[1], self.output_dim)

class LayerNormalization(Layer):

    def __init__(self, **kwargs):
        super(LayerNormalization, self).__init__(**kwargs)

    def call(self, inputs):
        #epsilon = np.float32(1e-8)
        epsilon = K.epsilon()

        mu = K.mean(inputs, axis=-1, keepdims=True)
        sigma = K.sqrt(K.var(inputs, axis=-1, keepdims=True)+ epsilon)
        inputs_normed = (inputs - mu) / (sigma + epsilon)

        return inputs_normed

    def compute_output_shape(self, input_shape):
        print(input_shape)
        return (input_shape[0], input_shape[1], input_shape[2])

class SelfAttentionNoParams(Layer):

    def __init__(self, output_dim, **kwargs):
        self.output_dim = output_dim
        super(SelfAttention, self).__init__(**kwargs)

    def build(self, input_shape):
        self.scaling_factor = np.sqrt(self.output_dim)
        super(SelfAttention, self).build(input_shape)  # Be sure to call this somewhere!

    def call(self, input_seq):
        query_seq = K.dot(input_seq, self.w_q)
        key_seq = K.dot(input_seq, self.w_k)
        value_seq = K.dot(input_seq, self.w_v)

        weights = K.batch_dot(query_seq, K.permute_dimensions(key_seq, (0,2,1))) # TODO: batch_dot? reshape?
        weights = activations.softmax(weights / self.scaling_factor)
        reweighted_values = K.batch_dot(weights, value_seq)
        return reweighted_values

    def compute_output_shape(self, input_shape):
        return (input_shape[0], input_shape[1], self.output_dim)